# Documentation string. I wonder what ever refers to it? 

__doc__ = """ZMailIn product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.
"""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.3.1' 

# Imports go here.  Fosters, Carling etc. 

from Products.ZCatalog.CatalogAwareness import CatalogAware # Makes messages catalog aware
from Products.ZCatalog import ZCatalog
from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

from OFS.SimpleItem import Item
from Acquisition import Implicit
import AccessControl.Role

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addZMailInForm = HTMLFile('ZMailInAdd', globals()) 

def manage_addZMailIn(self, id, title='', email='', method='', REQUEST=None):
    """Add a ZMailIn to a folder."""
    self._setObject(id, ZMailIn(id, title, email, method, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

"""Step #2: Provide the class for your ZMailIn object."""

class ZMailIn(
    CatalogAware,           # CatalogAware (Which is what it says)
    Item,                  # A simple Principia object. Not Folderish. 
    Persistent,            # Make us persistent. Yaah! 
    Implicit,              # Uh, whatever. 
    AccessControl.Role.RoleManager, # Security manager. 
    ZCatalog.ZCatalog
    ): 
    """ZMailIn object. """
        
    meta_type = 'ZMailIn Client'

    manage_options = ( # what management options are there?
	{'label': 'Edit',       'action': 'manage_main'},
	{'label': 'View',       'action': ''}, # defaults to index_html
	{'label': 'Security',   'action': 'manage_access'},
        ) 

    __ac_permissions__=( # what permissions make sense for us? 
	('View management screens', ('manage_tabs','manage_main')),
	('Change permissions',      ('manage_access',)           ),
	('Change ZMailIn'     ,     ('manage_edit',)             ),
	('View ZMailIn',            ('',)                        ),
	)

    def __init__(self, id, title='', email='', method='',REQUEST=None): 
        """initialise a new instance of ZMailIn"""
        import string, os
        
        self.id = id
        self.title = title
        self.email = email
        self.method = method
        self.default_catalog='GlobalZMailInCatalog'
        self.index_object()

    index_html = HTMLFile('index', globals()) # View Interface
    
    manage_main = HTMLFile('ZMailInEdit', globals()) # Management Interface
    
    def manage_edit(self, title, email, method, REQUEST=None): 
    	""" Edit """
        self.title = title
        self.email = email
        self.method = method
        self.default_catalog='GlobalZMailInCatalog'
        self.index_object()

	if REQUEST is not None:
	    return MessageDialog(
		title = 'Edited',
		message = "Properties for <I>%s</I> changed." % self.id,
		action = './manage_main',
		)
	
    def postZMailMessage(self, file, id='temp', title='', subject='', date='Tue, 13 Jun 2000 09:34:08 +0100', to='', sender='', replyto='', body='', headers='',REQUEST=None):
        """ 
        This method will create a dictionary containing all the information
	  about the Mail Message
	  The dictionary is then handed to the method stated in self.method for processing
        """
	import rfc822, multifile, StringIO
	
	theMail = {}
	theMail['id']=id
	theMail['title']=title
	theMail['subject']=subject
	theMail['date']=date
	theMail['to']=to
	theMail['sender']=sender
	theMail['replyto']=replyto
	theMail['headers']=headers
	theMail['attachments']={}

	self=self.this()
	parent = self.aq_parent
	
	mf = multifile.MultiFile(StringIO.StringIO(file),0)
	unpack_part(theMail,mf,1)

	REQUEST.set('theMail',theMail)
	#self.testMailIn(self,REQUEST)

	exec 'debug = parent.%s(self,REQUEST)' % (self.method,)
	#debug
	df = open("DebugFile.html",'w')
	df.write(debug)
	df.close()

	# xmlrpc throws a paddy if you don't return a string, so here goes
	return 'OK!'
	
def unpack_part(theMail, file, partno):
    	""" Will unpack a multipart message in the MIME format and enter it in to
    	the dictionary theMail"""
    	
    	import mimetools, multifile, sys, StringIO
    	
    	headers = mimetools.Message(file)
    	type = headers.getmaintype()
    	if type == 'multipart':
    		boundary = headers.getparam("boundary")
    		file.push(boundary)
    		file.readlines()
    		while not file.last:
    			file.next()
    			partno = partno + 1
    			unpack_part(theMail, file, partno)
    		file.pop()
    		return
    		
    	name = headers.getparam("name")
    	subtype = headers.getsubtype()
	if type=='text' and subtype=='plain' and not name:
		theMail['body'] = file.read()
    	else:
		if not name:
	    		name = 'part%d' % (partno,)
	    		name = name + '-'+subtype

		encoding = headers.getencoding()
    	
    	
 		if encoding == '7bit':
			outfile = StringIO.StringIO()
			mimetools.copyliteral(file,outfile)
		else:
			outfile = StringIO.StringIO()
			mimetools.decode(file,outfile,encoding)
    	
		theMail['attachments'][name]=outfile.getvalue()
		outfile.close()
