# Documentation string. I wonder what ever refers to it? 

__doc__ = """ZMailIn initialization module. 

This module is used by App.ProductContext (or something else buried deep 
in the guts of Zope which ends up calling App.ProductContext) to provide
product meta-data and perform any run-time initialization of a product 
that happens to be required. Since this product is ZMailIn, all we're doing 
is defining the meta-data.

The product itself does NOT live in here. That's in ZMailIn.py.
"""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.3.1'

# Step #1: import your Python class so we can refer to it later

import ZMailIn
from Products.ZCatalog import ZCatalog
from Products.ExternalMethod import ExternalMethod

# Step #2: define the initialize() function. 

def initialize(context): 
    """Initialize the ZMailIn product.
        
    registerClass() can accept a lot more arguments than we're providing 
    here."""
    
    """ This bit adds a Catalog to the root of zope for cataloging
        instances of ZMailIn """

    try: 
	app = context._ProductContext__app
    	proddir = app # Change this to place Global catalog somewhere different
    	
	if hasattr(proddir, 'GlobalZMailInCatalog'):
		globalcat = getattr(proddir, 'GlobalZMailInCatalog', None)
	else:
		oglobalcat = ZCatalog.ZCatalog('GlobalZMailInCatalog','Global ZMailIn Catalog')
		proddir._setObject('GlobalZMailInCatalog', oglobalcat)
		get_transaction().note('Added GlobalZMailInCatalog')
		get_transaction().commit()
		globalcat = getattr(proddir, 'GlobalZMailInCatalog', None)
		globalcat._catalog.addColumn('email')
		globalcat._catalog.addIndex('email','FieldIndex')
    except:
    	import traceback
    	traceback.print_exc()

    """ This bit adds an External Method to the Products/ZMailIn
        directory.
        This External Method is used by xmlrpc to Query the 
	  GlobalZMailInCatalog """

    try: 
    	proddir = app.Control_Panel.Products.ZMailIn
    	
	if hasattr(proddir, 'getClientLocation'):
		exMeth = getattr(proddir, 'getClientLocation', None)
	else:
		oexMeth = ExternalMethod.ExternalMethod('getClientLocation','Used by xmlrpc. (IMPORTANT)', 'ZMailInExtensions', 'getClientLocation')
		proddir._setObject('getClientLocation', oexMeth)
		get_transaction().note('Added External Method')
		get_transaction().commit()
		globalcat = getattr(proddir, 'getClientLocation', None)
    except:
    	import traceback
    	traceback.print_exc()

        """Try to register the product."""        
    try:
        context.registerClass(
            
            ZMailIn.ZMailIn,                   # Which is the addable bit? 
            
            constructors = (                   # The first of these is called 
                ZMailIn.manage_addZMailInForm, # when someone adds the product; 
                ZMailIn.manage_addZMailIn),    # the second is named here so we 
                                               # can give people permission to 
                                               # call it. 
                                             
            icon = 'mailin.png'
            )
       
    except:
        """If you can't register the product, tell someone. 
        
        Zope will sometimes provide you with access to "broken product" and
        a backtrace of what went wrong, but not always; I think that only 
        works for errors caught in your main product module. 
        
        This code provides traceback for anything that happened in 
        registerClass(), assuming you're running Zope in debug mode."""
        
        import sys, traceback, string
        type, val, tb = sys.exc_info()
        sys.stderr.write(string.join(traceback.format_exception(type, val, tb), ''))
        del type, val, tb
