# Documentation string. 
__doc__ = """ZMailMessage product module.

		This module provides the methods and classes that Zope needs for the product. 
		__init__.py is responsible for registering these methods and classes.
		"""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.3.1'

# Imports go here

from Products.ZCatalog.CatalogAwareness import CatalogAware   # Makes messages catalog aware
from Products.ZCatalog import ZCatalog
from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

from OFS.Folder import Folder
from Acquisition import Implicit
import AccessControl.Role

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addZMailMessageForm = HTMLFile('ZMailMessageAdd', globals()) 

def manage_addZMailMessage(self, id, title='', subject='', date='', to='', sender='', replyto='', body='', headers='', REQUEST=None):
    """Add a ZMailMessage to a folder."""
    self._setObject(id, ZMailMessage(id, title, subject, date, to, sender, replyto, body, headers))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

"""Step #2: Provide the class for your ZMailMessage object."""

class ZMailMessage(
    CatalogAware,          # CatalogAware (Which is what it says)
    Folder,                # Folderish. 
    #Persistent,            # Make us persistent. Yaah! 
    #Implicit,              # Uh, whatever. 
    #AccessControl.Role.RoleManager, # Security manager. 
    ZCatalog.ZCatalog
    ): 
    """ZMailMessage object."""
        
    meta_type = 'ZMailMessage'

    manage_options = ( # the management options available
    	{'label': 'Contents',	'action': 'manage_main'},
	{'label': 'View',       'action': ''}, # defaults to index_html
	{'label': 'Properties',       'action': 'manage_propertiesForm'},
	{'label': 'Security',   'action': 'manage_access'},
        ) 

    __ac_permissions__=( # what permissions make sense for us? 
	('View management screens', ('manage_tabs','manage_main')),
	('Change permissions',      ('manage_access',)           ),
	('Change ZMailMessages',    ('manage_edit',)             ),
	('View ZMailMessage',       ('',)                        ),
	)

    def __init__(self, id, title='', subject='', date='', to='', sender='', replyto='', body='', headers=''): 
        """initialise a new instance of ZMailMessage"""
        #ZCatalog.ZCatalog.__init__(self,id,title)
        self.id = id
        self.title = title
        self.subject = subject
        self.date = date
        self.to = to
        self.sender = sender
        self.replyto = replyto
        self.body = body
        self.headers = headers
        self.default_catalog='ZMailInCatalog'
        self.index_object()
        #self._catalog.addIndex('body','TextIndex')
        #self._catalog.addColumn('body')

    index_html = HTMLFile('index', globals()) # View Interface
    
    manage_propertiesForm = HTMLFile('ZMailMessageEdit', globals()) # Properties Interface

    def manage_edit(self, title, subject, date, to, sender, replyto, body, headers, REQUEST=None): 
        """After editing the Object this function is called."""
        self.title = title
        self.subject = subject
        self.date = date
        self.to = to
        self.sender = sender
        self.replyto = replyto
        self.body = body
        self.headers = headers
        self.default_catalog='ZMailInCatalog'
        self.reindex_object()
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Edited',
		message = "Properties for <I>%s</I> changed." % self.id,
		action = './manage_main',
		)
		
    def setbody(self, body):
    	self.body = body
        self.default_catalog='ZMailInCatalog'
        self.reindex_object()
